//*****************************************************************************
//
//	Routinen fuer das Lesen und Schreiben in den EEPROM des ATmega
//
//*****************************************************************************
#include <avr/io.h>
#include "WATCHDOG.H"

// 16MHz * 1ms
#define ZK  16000

//*****************************************************************************
//	Schreiben in EEPROM
//  Input:	1 DB, ADRESSE
//	Output:	-
//*****************************************************************************

void writeb_eeprom(unsigned int adr, unsigned char uc_data)
{
	while((EECR & 0x02) != 0)
	{
	   watchdog_reset();	
	}
	
	SREG &= 0x7F;	//global INT aus  
	EEARH = (unsigned char)(adr >> 8);	//Highteil adr
	EEARL = (unsigned char) adr;		//Lowteil  adr
	EEDR  = uc_data;			   		//Daten
	EECR  = 0x04;     					//write + Strobe Master
	EECR  = 0x06;						//write + Strobe
	SREG |= 0x80;						//global INT an   
}


//*****************************************************************************
//	Schreiben in EEPROM
//  Input:	1 DW, ADRESSE
//	Output:	-
//*****************************************************************************

void writew_eeprom(unsigned int adr, unsigned int ui_data)
{
	writeb_eeprom(adr,  (unsigned char)(ui_data >> 8));
	writeb_eeprom(adr+1,(unsigned char) ui_data);
}                        


//*****************************************************************************
//	Lesen aus EEPROM
//  Input:	ADRESSE
//	Output:	1 DB
//*****************************************************************************

unsigned char readb_eeprom(unsigned int adr)
{
	while((EECR & 0x02) != 0)
	{
	   watchdog_reset();	
	}
	EEARH = (unsigned char)(adr >> 8);	//Highteil adr
	EEARL = (unsigned char) adr;		//Lowteil  adr
	EECR = EECR | 0x01; 				//read + Strobe
	return(EEDR);						//Daten
}


//*****************************************************************************
//	Lesen aus EEPROM
//  Input:	ADRESSE
//	Output:	1 DW
//*****************************************************************************

unsigned int readw_eeprom(unsigned int adr)
{
unsigned int x;

	x = ((unsigned int) readb_eeprom(adr) << 8);
	x = x | (unsigned int) readb_eeprom(adr+1);
	return(x);
}
