//*****************************************************************************
//
//	Kommandointerpreter0 fuer UART0
//
//*****************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <avr/wdt.h> 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "watchdog.h"
#include "kommand0.h"
#include "proto0.h"
#include "EEPROM.H"

extern unsigned char uc_motorstate;
extern unsigned int  ui_pwm,ui_pwminc,ui_speed,ui_minpwm;
extern unsigned long ul_count,ul_weg,ul_brakeinc,ul_kalibrierwert;

char c_txstr0[80];
unsigned int  ui_x;
char c_tmp[20];


//*****************************************************************************
//	Kommand - Kommandointerpreter
//  Input:	Befehlsstring
//	Output:	-
//*****************************************************************************
void Kommandointerpreter0(char * pc_str)
{
unsigned char uc_ok = 0;

   watchdog_reset();
   if (strncmp(pc_str,"START",5) == 0)   uc_ok = BEFEHL0_Start(pc_str+5); 
   if (strncmp(pc_str,"HALT",4) == 0)    uc_ok = BEFEHL0_Halt(pc_str+4);

   watchdog_reset();
   if (strncmp(pc_str,"SK:",3) == 0)     uc_ok = BEFEHL0_SetKalibrierwert(pc_str+3);
   if (strncmp(pc_str,"GK",2) == 0)      uc_ok = BEFEHL0_GetKalibrierwert(pc_str+2);

   watchdog_reset();
   if (strncmp(pc_str,"SA:",3) == 0)     uc_ok = BEFEHL0_SetBeschleunigung(pc_str+3);
   if (strncmp(pc_str,"GA",2) == 0)      uc_ok = BEFEHL0_GetBeschleunigung(pc_str+2);

   watchdog_reset();
   if (strncmp(pc_str,"SV:",3) == 0)     uc_ok = BEFEHL0_SetGeschwindigkeit(pc_str+3);
   if (strncmp(pc_str,"GV",2) == 0)      uc_ok = BEFEHL0_GetGeschwindigkeit(pc_str+2);

   watchdog_reset();
   if (strncmp(pc_str,"DOSIS:",6) == 0)  uc_ok = BEFEHL0_Dosis(pc_str+6);
   if (strncmp(pc_str,"GC",2) == 0)		 uc_ok = BEFEHL0_GetCount(pc_str+2);

   watchdog_reset();
   if (strncmp(pc_str,"SM:",3) == 0)     uc_ok = BEFEHL0_SetMinPWM(pc_str+3);
   if (strncmp(pc_str,"GM",2) == 0)      uc_ok = BEFEHL0_GetMinPWM(pc_str+2);
   
   watchdog_reset();
   if (strncmp(pc_str,"GS",2) == 0)      uc_ok = BEFEHL0_GetStatus(pc_str+2);
   
   watchdog_reset();
   
   if (!uc_ok) BEFEHL0_ERROR("");
}


//*****************************************************************************
//	Kommand - Kommando OK  
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_OK(char * pc_str)
{
   watchdog_reset();	
   strcpy(c_tmp,"OK");	
   sende_UART0(c_tmp);	
   return(1);
}


//*****************************************************************************
//	Kommand - Kommando ERROR  
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_ERROR(char * pc_str)
{
   watchdog_reset();
   strcpy(c_tmp,"ERROR");
   sende_UART0(c_tmp);
   return(1);
}


//*****************************************************************************
//	Kommand - Kommando Start
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_Start(char * pc_str)
{
   watchdog_reset();
   if (uc_motorstate == MOTORSTATE_STOP)
    {
	   ul_weg = -1;	
	   uc_motorstate = MOTORSTATE_START;	
       return(1);		
	}
    else return(0);
}


//*****************************************************************************
//	Kommand - Kommando Halt
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_Halt(char * pc_str)
{
   watchdog_reset();
   if (uc_motorstate != MOTORSTATE_STOP)
    {
	   ui_pwm = 0;  //sofort stoppen	
	   uc_motorstate = MOTORSTATE_STOP;
	   return(1);
    }
    else return(0);
}

//*****************************************************************************
//	Kommand - Kommando SetKalibrierwert (inc/ml)
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_SetKalibrierwert(char * pc_str)
{
   ul_kalibrierwert = atol(pc_str);
   watchdog_reset();
   writew_eeprom(0x0004,((unsigned long) ul_kalibrierwert >> 16));
   writew_eeprom(0x0006,ul_kalibrierwert);
   watchdog_reset();
   writew_eeprom(0x0000,0x0000);
   return(1);
}


//*****************************************************************************
//	Kommand - Kommando GetKalibrierwert (inc/ml)
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_GetKalibrierwert(char * pc_str)
{
   strcpy(c_txstr0,"GK:");
   strcpy(c_tmp,"");
   watchdog_reset();
   ltoa(ul_kalibrierwert,c_tmp,10);
   strcat(c_txstr0,c_tmp);
   watchdog_reset();
   sende_UART0(c_txstr0);	
   return(1);
}


//*****************************************************************************
//	Kommand - Kommando SetBeschleunigung
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_SetBeschleunigung(char * pc_str)
{
   ui_x = atoi(pc_str);
   if (ui_x < 0x0100)
    {		
       ui_pwminc = ui_x;
       watchdog_reset();
       writew_eeprom(0x0002,ui_pwminc);
       watchdog_reset();
       writew_eeprom(0x0000,0x0000);
       return(1);
	}
	else return(0);
}


//*****************************************************************************
//	Kommand - Kommando GetBeschleunigung
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_GetBeschleunigung(char * pc_str)
{
   strcpy(c_txstr0,"GA:");
   strcpy(c_tmp,"");
   watchdog_reset();
   itoa(ui_pwminc,c_tmp,10);
   strcat(c_txstr0,c_tmp);
   watchdog_reset();
   sende_UART0(c_txstr0);	
   return(1);
}


//*****************************************************************************
//	Kommand - Kommando SetGeschwindigkeit
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_SetGeschwindigkeit(char * pc_str)
{
   ui_x = atoi(pc_str);
   if (ui_x < 0x0100)
    {
	   ui_speed = ui_x;
       watchdog_reset();
       writew_eeprom(0x0008,ui_speed);
       watchdog_reset();
       writew_eeprom(0x0000,0x0000);
       return(1);
	}
	else return(0);
}


//*****************************************************************************
//	Kommand - Kommando GetGeschwindigkeit
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_GetGeschwindigkeit(char * pc_str)
{
   strcpy(c_txstr0,"GV:");
   strcpy(c_tmp,"");
   watchdog_reset();
   itoa(ui_speed,c_tmp,10);
   strcat(c_txstr0,c_tmp);
   watchdog_reset();
   sende_UART0(c_txstr0);	
   return(1);
}


//*****************************************************************************
//	Kommand - Kommando SetMinPWM
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_SetMinPWM(char * pc_str)
{
   ui_x = atoi(pc_str);
   if (ui_x < 0x0100)
    {
	   ui_minpwm = ui_x;
       watchdog_reset();
       writew_eeprom(0x000A,ui_minpwm);
       watchdog_reset();
       writew_eeprom(0x0000,0x0000);
       return(1);
	}
	else return(0);
}


//*****************************************************************************
//	Kommand - Kommando GetMinPWM
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_GetMinPWM(char * pc_str)
{
   strcpy(c_txstr0,"GM:");
   strcpy(c_tmp,"");
   watchdog_reset();
   itoa(ui_minpwm,c_tmp,10);
   strcat(c_txstr0,c_tmp);
   watchdog_reset();
   sende_UART0(c_txstr0);	
   return(1);
}


//*****************************************************************************
//	Kommand - Kommando GetStatus
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_GetStatus(char * pc_str)
{
   watchdog_reset();
   if (uc_motorstate == MOTORSTATE_STOP) strcpy(c_txstr0,"GS:0");
	else strcpy(c_txstr0,"GS:1");

   watchdog_reset();
   sende_UART0(c_txstr0);	
   return(1);
}


//*****************************************************************************
//	Kommand - Kommando Dosis
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_Dosis(char * pc_str)
{
float f_menge;

   f_menge = atof(pc_str);
   watchdog_reset();
   f_menge = f_menge * ul_kalibrierwert;
   watchdog_reset();
   ul_weg = roundf(f_menge);
   watchdog_reset();

   if (uc_motorstate == MOTORSTATE_STOP)
    {
	   uc_motorstate = MOTORSTATE_START;
	   return(1);
	}
	else return(0);
}


//*****************************************************************************
//	Kommand - Kommando GetCount
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL0_GetCount(char * pc_str)
{
   strcpy(c_txstr0,"GC:");
   strcpy(c_tmp,"");
   watchdog_reset();
   ltoa(ul_weg,c_tmp,10);
   strcat(c_txstr0,c_tmp);
   watchdog_reset();
   sende_UART0(c_txstr0);
   return(1);
}
