//*****************************************************************************
//
//	UART des ATmega Interrupt gesteuert
//
//*****************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdlib.h>
#include <avr/wdt.h>
#include <string.h>
#include "UART0.h"
#include "watchdog.h"


#define BUF0_LAENGE  128

//#pragma interrupt_handler UART0_RX_INT:UART0_RXC
//#pragma interrupt_handler UART0_DRE_INT:UART0_DRE

char c_RXBUF0[BUF0_LAENGE+1];
char c_TXBUF0[BUF0_LAENGE+1];
  
unsigned int ui_RX0cnt,ui_TX0cnt,ui_RX0wr,ui_TX0wr,ui_RX0rd,ui_TX0rd;
unsigned char uc_cnt = 0;


//*****************************************************************************
//	RX - INT
//*****************************************************************************

ISR( USART_RX_vect)
{
   UART0_RX_INT();
} 




void UART0_RX_INT(void)		//RXC
{
   c_RXBUF0[ui_RX0wr]  = UDR0;
   ui_RX0wr++;
  
   if (ui_RX0wr >= BUF0_LAENGE) ui_RX0wr = 0;
   ui_RX0cnt++; 
}

//*****************************************************************************
//	TX - INT  USART_UDRE_vect
//*****************************************************************************
ISR (USART_UDRE_vect )
{
   UART0_DRE_INT();
}



void UART0_DRE_INT(void)                 //TX-Interrupt
{
  if (ui_TX0cnt)
   {
      UDR0 = c_TXBUF0[ui_TX0rd];
	  ui_TX0rd++;
	  if (ui_TX0rd >= BUF0_LAENGE) ui_TX0rd = 0;
	  if (ui_TX0cnt > 0) ui_TX0cnt--; 
   }
//   else UCSR0B &= 0xDF;
}


//*****************************************************************************
//	UART - Initialisierung
//	Input:	-
//	Output:	-
//*****************************************************************************

void UART0_INIT(void)
{
unsigned char x;
unsigned int i;

   SREG &= 0x7F;				//global INT aus
	                   
   ui_RX0cnt = 0;
   ui_TX0cnt = 0;
   ui_RX0wr  = 0;
   ui_TX0wr  = 0;
   ui_RX0rd  = 0;
   ui_TX0rd  = 0;
	
   for(i=0;i<BUF0_LAENGE+1;i++)
	{
	   c_TXBUF0[i] = 0;
	   c_RXBUF0[i] = 0;
       watchdog_reset();
	}
	
	// DDRD 	= 0xFE;  
	// PORTD 	= 0x00; 	

   UCSR0A = 0;    //Int Flag loeschen
   UCSR0B = 0;
   UCSR0C = 0x0E;

   UBRR0  = 103;				// 9k6 Baudrate (103 bei 16MHz)
//	UBRR0  = 51;				//19k2 Baudrate ( 51 bei 16MHz)
   
   
//   UBRR0  = 119;				// 9k6 Baudrate (119 bei 18,432MHz)
//	UBRR0  = 59;				//19k2 Baudrate ( 59 bei 18,432MHz)
//	UBRR0  = 19;				//57k6 Baudrate ( 19 bei 18,432MHz)

//	UBRR0  = 129;				// 9k6 Baudrate (129 bei 20MHz)
//	UBRR0  = 64;				//19k2 Baudrate ( 64 bei 20MHz)


   UCSR0B = 0x98;				//RX & TX der UART an, Interrupts enabled	
	
   x = UDR0;

   while((UCSR0A & 0x20) == 0) //UDRE == 0 ?
    {
       watchdog_reset();
    }

   while((UCSR0A & 0x40) == 0x40) //TXC  == 1 ?
    {
       watchdog_reset();
    }

   UDR0 = 0x0D;					//sende CR, damit TXC flag setzen
   SREG |= 0x80;				//global INT an
}
 
//*****************************************************************************
//	UART - Test, ob Zeichen empfangen
//	Input:	-
//	Output:	1 ja , 0 nein
//*****************************************************************************

unsigned char UART0_RXTest(void)
{
   if (ui_RX0cnt) return(1); 
   	 else return(0);
}


//*****************************************************************************
//	UART - Zeichen senden
//	Input:	char  Zeichen
//	Output:	-
//*****************************************************************************

void UART0_TX(char c_x)
{
   while(ui_TX0cnt > 0) //warte bis Puffer leer
   {
      watchdog_reset();
   }

   SREG &= 0x7F;  //INTs sperren  

   c_TXBUF0[ui_TX0wr] = c_x;
   ui_TX0wr++;
   if (ui_TX0wr >= BUF0_LAENGE) ui_TX0wr = 0;
   ui_TX0cnt++; 
   if (ui_TX0cnt >= BUF0_LAENGE) ui_TX0cnt = 0; 

   SREG |= 0x80;  //INTs freigeben  

   UCSR0B |= 0x20; 	     
}


//*****************************************************************************
//	UART - String senden
//	Input:	char* String 
//	Output:	-
//*****************************************************************************

void UART0_TXStr(char * pc_str)
{
   while(ui_TX0cnt > 0) //warte bis Puffer leer
   {
      watchdog_reset();
   }

   while (*pc_str != '\0')
   {
      c_TXBUF0[ui_TX0wr] = *pc_str;
      ui_TX0wr++;
      if (ui_TX0wr >= BUF0_LAENGE) ui_TX0wr = 0;
      SREG &= 0x7F;  //INTs sperren  
      ui_TX0cnt++; 
      if (ui_TX0cnt >= BUF0_LAENGE) ui_TX0cnt = 0;
      SREG |= 0x80;    //INTs freigeben  
  	  pc_str++; 
      watchdog_reset();
   }
   
  UCSR0B |= 0x20; 	 
}


//*****************************************************************************
//	UART - Zeichen empfangen
//	Input:	- 
//	Output:	char  Empfangszeichen
//*****************************************************************************

char UART0_RX(void)
{      
char uc_x;
         
   while(ui_RX0cnt == 0)		//warte auf Zeichen
   {
      watchdog_reset();
   }

   uc_x = c_RXBUF0[ui_RX0rd];
  
   SREG &= 0x7F;  //INTs sperren  
   ui_RX0rd++;
   if (ui_RX0rd >= BUF0_LAENGE) ui_RX0rd = 0;
   if (ui_RX0cnt > 0) ui_RX0cnt--;  
   SREG |= 0x80;  //INTs freigeben  
  
   return(uc_x);
}


