//****************************************************************************
//File name:	main.c
//Data:			20/01/2022
//Version:		1
//Company:		
//Details: 		Peristaltikpumpe
//				Language C: GCC compiler
//				Fosc = 16 MHz
// Fuse: 0xFD 0xDB 0xFF             
// DIV8 disable
//
//Author:	    N.Paasche (Dipl.-Ing.(FH) Ing.)	
//****************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdlib.h>
#include <avr/wdt.h>
#include <avr/common.h>
#include "MCUinit.h"
#include "UART0.h"
#include "PROTO0.h"
#include "KOMMAND0.H"
#include "watchdog.h"
#include "timer0.h"
#include "PWM.H"
#include "EEPROM.H"

#define tk_100ms			100

unsigned char uc_motorstate;
unsigned int  ui_pwm,ui_pwminc,ui_speed,ui_minpwm;
unsigned long ul_count,ul_weg,ul_brakeinc,ul_kalibrierwert;

//*****************************************************************************
//	Main - PortInteruptroutine
//  Input:	-
//	Output:	-
//*****************************************************************************
ISR(PCINT2_vect) 
{
   if (ul_count < 0xFFFFFFFF) ul_count++;  
}


//*****************************************************************************
//	Main - Startroutine
//  Input:	-
//	Output:	-
//*****************************************************************************
int main(void)
{
   MCUinit();					// MCU initialisieren
   timer0_init();
   pwm1_init();
   UART0_INIT();
  
   watchdog_reset();   
   watchdog_set(0x07);
   watchdog_enable();

   timer0_setcnt1(0); 
   pwm1_setWert(0x00);   	

   DDRD &= 0xF6;
   PORTD |= 0x08;
   PCMSK2 |= 0x08; //PCINT 19 PD3
   PCICR |= 0x04;  //PCIE2
   PCIFR= 0x07; 	


	
   uc_motorstate = 0;
   ui_pwm = 0;
   ui_pwminc = 1;
   ul_count = 0;
   ul_weg = 0;
   ul_brakeinc = 0;
   ul_kalibrierwert = 1;
   ui_speed = 0x00FF;
   ui_minpwm = 0x10;
   
   watchdog_reset();
   
   if (readw_eeprom(0x0000) == 0x0000)
    {
	   ui_pwminc = readw_eeprom(0x0002);	
	   ul_kalibrierwert = ((unsigned long) readw_eeprom(0x0004) << 16) | (unsigned long) readw_eeprom(0x0006);	
	   ui_speed = readw_eeprom(0x0008);
	   ui_minpwm = readw_eeprom(0x00A);
	}
   
   //Endlosschleife
   while (1)
    {
	   watchdog_reset();

	   if (UART0_RXTest())
	    {
		   watchdog_reset();
		   receive_uart0();
	    } //UART

	   watchdog_reset();
	   if (!timer0_getcnt1())
	    {
		   timer0_setcnt1(tk_100ms);
		   
		   switch(uc_motorstate)
		   {
		   case MOTORSTATE_STOP:	ul_count = 0;
									ul_brakeinc = 0;
									ui_pwm = 0;
									pwm1_setWert(ui_pwm);
									break;
									
		   case MOTORSTATE_START:	ui_pwm += ui_pwminc;
									if (ui_pwm < ui_minpwm)
									 {
										ui_pwm = ui_minpwm; 
									 }
		   
									if (ui_pwm > 0xFF)
									 {
										ui_pwm = 0xFF;
										uc_motorstate = MOTORSTATE_RUN;
									 }
									
									if (ui_pwm >= ui_speed)
									 {
										uc_motorstate = MOTORSTATE_RUN;										 
									 }
									 
									if (ul_count >= (ul_weg /2)) 
									 {
										ul_brakeinc = ul_count; 
									 }
									 
									if (ul_count >= (ul_weg - ul_brakeinc))
									 {
										uc_motorstate = MOTORSTATE_BRAKE;
									 }						 
									break;
									
		   case MOTORSTATE_RUN:		if (ul_count >= (ul_weg - ul_brakeinc))
									 {
										uc_motorstate = MOTORSTATE_BRAKE;
									 }
									break;
									
		   case MOTORSTATE_BRAKE:	if (ui_pwm > ui_pwminc)
									 {
										ui_pwm -= ui_pwminc;
									 }
									 
									 if (ui_pwm < ui_minpwm)
									  {
										 ui_pwm = ui_minpwm;  
									  }
									 
									 if (ul_count >= ul_weg) 
									  {
										 ui_pwm = 0;									
										 uc_motorstate = MOTORSTATE_STOP; 
										 pwm1_setWert(ui_pwm);
										 ul_weg = ul_count;
									  } 
									break;																									   
		   } //switch
		  
		   watchdog_reset();
		   if (uc_motorstate != MOTORSTATE_STOP)
		    {
			   pwm1_setWert((unsigned int) ui_pwm);	
			}	
		}
    } //while
}	



