//*****************************************************************************
//
//	Timer0
//
//*****************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include "timer0.h"
#include "watchdog.h"

#define   tk_1ms   6

//#pragma interrupt_handler Timer0_int:iv_TIM0_OVF

unsigned long ul_zeit = 0, ul_timercnt1 = 0, ul_timercnt2 = 0;

//*****************************************************************************
//	Timer0-Interrupt OVERFLOW
//  Input:	-
//	Output:	-
//*****************************************************************************
ISR(TIMER0_OVF_vect)
{
   TCNT0 = tk_1ms;  
   if (ul_zeit) ul_zeit--;
   if (ul_timercnt1) ul_timercnt1--;	   	    
   if (ul_timercnt2) ul_timercnt2--;	   	    
}


//*****************************************************************************
//	Timer0 - Init
//  Input:	-
//	Output:	-
//*****************************************************************************
void timer0_init(void)
{
   SREG &= 0x7F;
   TIFR0 |= 0x07; //Overflow Flag reset
   
   TIMSK0 |= 0x01; //Timer0 OVR Int enable
   TCCR0A = 0;     //keine Ausgaenge mit Timer schalten 
   TCCR0B = 0x03;  //Vorteiler div64 0x03
   TCNT0 = tk_1ms;
   
   ul_zeit = 0;
   ul_timercnt1 = 0;   
   ul_timercnt2 = 0;   
   SREG |= 0x80;   //global Int enable   
}


//*****************************************************************************
//	Timer0 - SetTimerCnt1
//  Input:	cnt
//	Output:	-
//*****************************************************************************
void timer0_setcnt1(unsigned long ul_x)
{
   ul_timercnt1 = ul_x;
}


//*****************************************************************************
//	Timer0 - GetTimerCnt1
//  Input:	-
//	Output:	cnt
//*****************************************************************************
unsigned long timer0_getcnt1(void)
{
   return(ul_timercnt1);
}


//*****************************************************************************
//	Timer0 - SetTimerCnt2
//  Input:	cnt
//	Output:	-
//*****************************************************************************
void timer0_setcnt2(unsigned long ul_x)
{
   ul_timercnt2 = ul_x;
}


//*****************************************************************************
//	Timer0 - GetTimerCnt2
//  Input:	-
//	Output:	cnt
//*****************************************************************************
unsigned long timer0_getcnt2(void)
{
   return(ul_timercnt2);
}


//*****************************************************************************
//	Timer - DELAY
//  Input:	Zeitangabe (zeit in ms)
//	Output:	-
//*****************************************************************************
void timer0_delay(unsigned long x)
{
 	 ul_zeit = x;
	 while(ul_zeit > 0)		  	   //warte bis zeit abgelaufen
	 {
	    watchdog_reset();
	 }
}
